@@ -1,5 +1,10 @@
 """Define configuration objects.
 """
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import errno
 import mimetypes
 import os
@@ -12,7 +17,6 @@ from collections import defaultdict
 import aspen
 import aspen.logging
 from aspen import execution
-from aspen.hooks import Hooks
 from aspen.configuration import parse
 from aspen.configuration.exceptions import ConfigurationError
 from aspen.configuration.options import OptionParser, DEFAULT
@@ -26,8 +30,8 @@ from aspen.utils import ascii_dammit
 
 KNOBS = \
     { 'configuration_scripts': (lambda: [], parse.list_)
-    , 'network_engine':     (u'cheroot', parse.network_engine)
-    , 'network_address':    ( ((u'0.0.0.0', 8080), socket.AF_INET)
+    , 'network_engine':     ('cheroot', parse.network_engine)
+    , 'network_address':    ( (('0.0.0.0', 8080), socket.AF_INET)
                             , parse.network_address
                              )
     , 'project_root':       (None, parse.identity)
@@ -38,10 +42,10 @@ KNOBS = \
     # Extended Options
     # 'name':               (default, from_unicode)
     , 'changes_reload':     (False, parse.yes_no)
-    , 'charset_dynamic':    (u'UTF-8', parse.charset)
+    , 'charset_dynamic':    ('UTF-8', parse.charset)
     , 'charset_static':     (None, parse.charset)
-    , 'indices':            ( lambda: [u'index.html', u'index.json', u'index'] +
-                                      [u'index.html.spt', u'index.json.spt', u'index.spt']
+    , 'indices':            ( lambda: ['index.html', 'index.json', 'index'] +
+                                      ['index.html.spt', 'index.json.spt', 'index.spt']
                             , parse.list_
                              )
     , 'list_directories':   (False, parse.yes_no)
@@ -86,11 +90,11 @@ class Configurable(object):
         return out % (name, hydrated, context + name_in_context)
 
     def set(self, name, raw, from_unicode, context, name_in_context):
-        assert isinstance(raw, str), "%s isn't a bytestring" % name
-
         error = None
         try:
-            value = raw.decode('US-ASCII')
+            value = raw
+            if isinstance(value, str):
+                value = raw.decode('US-ASCII')
             hydrated = from_unicode(value)
         except UnicodeDecodeError, error:
             value = ascii_dammit(value)
@@ -265,9 +269,9 @@ class Configurable(object):
         for entrypoint in pkg_resources.iter_entry_points(group='aspen.renderers'):
             render_module = entrypoint.load()
             self.renderer_factories[entrypoint.name] = render_module.Factory(self)
-            aspen.log_dammit("Found plugin for renderer '%s'" % entrypoint.name) 
+            aspen.log_dammit("Found plugin for renderer '%s'" % entrypoint.name)
 
-        self.default_renderers_by_media_type = defaultdict(lambda: self.renderer_default) 
+        self.default_renderers_by_media_type = defaultdict(lambda: self.renderer_default)
 
         # mime.types
         # ==========
@@ -285,7 +289,7 @@ class Configurable(object):
         ENGINES = {}
         for entrypoint in pkg_resources.iter_entry_points(group='aspen.network_engines'):
             ENGINES[entrypoint.name] = entrypoint.load()
-        
+
         if self.network_engine in ENGINES:
             # found in a module
             Engine = ENGINES[self.network_engine].Engine
@@ -313,33 +317,6 @@ class Configurable(object):
         else:
             self.network_port = None
 
-        # hooks
-        self.hooks = Hooks()
-        self.hooks.startup = []
-        self.hooks.inbound_early = []
-        self.hooks.inbound_core = []
-        self.hooks.inbound_late = []
-        self.hooks.error_early = []
-        self.hooks.error_late = []
-        self.hooks.outbound = []
-        self.hooks.shutdown = []
-
-
-        # Set up core logic as hooks.
-        # ===========================
-        # This way, apps have fairly complete control over the request handling
-        # cycle. We don't have an error_core because error handling is more
-        # complicated.
-
-        self.reset_startup()
-        self.reset_inbound_early()
-        self.reset_inbound_core()
-        self.reset_inbound_late()
-        self.reset_error_early()
-        self.reset_error_late()
-        self.reset_outbound()
-        self.reset_shutdown()
-
         self.run_config_scripts()
         self.show_renderers()
 
@@ -371,7 +348,7 @@ class Configurable(object):
         # ========================================
         # The user gets self as 'website' inside their configuration scripts.
         default_cfg_filename = None
-	if self.project_root is not None:
+        if self.project_root is not None:
             default_cfg_filename = os.path.join(self.project_root, DEFAULT_CONFIG_FILE)
 
         for filepath in self.configuration_scripts:
@@ -385,15 +362,19 @@ class Configurable(object):
             try:
                 execfile(filepath, {'website': self})
             except IOError, err:
+                # Re-raise the error if it happened inside the script.
+                if err.filename != filepath:
+                    raise
+
                 # I was checking os.path.isfile for these, but then we have a
                 # race condition that smells to me like a potential security
                 # vulnerability.
 
-		## determine if it's a default configscript or a specified one
+                ## determine if it's a default configscript or a specified one
                 cfgtype = "configuration"
-		if filepath == default_cfg_filename:
-		    cfgtype = "default " + cfgtype
-		## pick the right error mesage
+                if filepath == default_cfg_filename:
+                    cfgtype = "default " + cfgtype
+                ## pick the right error mesage
                 if err.errno == errno.ENOENT:
                     msg = ("The %s script %s doesn't seem to exist.")
                 elif err.errno == errno.EACCES:
@@ -403,14 +384,14 @@ class Configurable(object):
                     msg = ("There was a problem reading the %s script %s:")
                     msg += os.sep + traceback.format_exc()
                 ## do message-string substitutions
-		msg = msg % (cfgtype, filepath)
-		## output the message
-		if not "default" in cfgtype:
+                msg = msg % (cfgtype, filepath)
+                ## output the message
+                if not "default" in cfgtype:
                    # if you specify a config file, it's an error if there's a problem
                    raise ConfigurationError(msg)
-	        else:
+                else:
                    # problems with default config files are okay, but get logged
-		   aspen.log(msg)
+                   aspen.log(msg)
             else:
                 aspen.log_dammit("Loading configuration file '%s' (possibly changing settings)" % filepath)
                 execution.if_changes(filepath)
@@ -418,13 +399,3 @@ class Configurable(object):
 
 
 
-    # Override these in subclasses to implement default logic.
-
-    def reset_startup(self): pass
-    def reset_inbound_early(self): pass
-    def reset_inbound_core(self): pass
-    def reset_inbound_late(self): pass
-    def reset_outbound(self): pass
-    def reset_error_early(self): pass
-    def reset_error_late(self): pass
-    def reset_shutdown(self): pass
